import React, { forwardRef, Ref, useImperativeHandle, useState } from 'react'
import { useTranslation } from 'react-i18next'

import {
  Box,
  Grid,
  Table,
  TableBody,
  TableCell,
  TableHead,
  TableRow,
} from '@mui/material'

import format from 'date-fns/format'
import i18n from 'i18next'
import { RegisterFormData } from './formData'
import { postJsonOpts } from '../../utils'
import { PersonFormMethods } from './personFormMethods'
import useOus from '../../hooks/useOus'
import useRoleTypes from '../../hooks/useRoleTypes'
import SubmitState from './submitState'


interface StepSummaryProperties {
  formData: RegisterFormData
  onSubmitSuccess: () => void
  onSubmitFailure: () => void
}

const StepSummary = forwardRef((props: StepSummaryProperties, ref: Ref<PersonFormMethods>) => {
    const { t } = useTranslation(['common'])
    const { formData, onSubmitSuccess, onSubmitFailure } = props
    const ous = useOus()
    const roleTypes = useRoleTypes()
    const [submitState, setSubmitState] = useState(SubmitState.NotSubmitted)

    const submit = () => {
      const payload = {
        first_name: formData.first_name,
        last_name: formData.last_name,
        email: formData.email,
        role: {
          type: formData.role_type,
          start_date: formData.role_start === null ? null : format(formData.role_start as Date, 'yyyy-MM-dd'),
          end_date: formData.role_end === null ? null : format(formData.role_end as Date, 'yyyy-MM-dd'),
          comments: formData.comment,
          orgunit_id: formData.ou_id,
        },
      }

      console.log('submitting', JSON.stringify(payload))
      fetch('/api/ui/v1/invite/', postJsonOpts(payload))
        .then((res) => {
          if (!res.ok) {
            onSubmitFailure()
            setSubmitState(SubmitState.SubmitFailure)
            return null
          }
          return res.text()
        })
        .then((result) => {
          if (result !== null) {
            console.log('result', result)
            onSubmitSuccess()
            setSubmitState(SubmitState.SubmitSuccess)
          }
        })
        .catch((error) => {
          console.log('error', error)
          onSubmitFailure()
          setSubmitState(SubmitState.SubmitFailure)
        })
    }

    function doSubmit() {
      return submit()
    }

    useImperativeHandle(ref, () => ({ doSubmit }))

    const selectedOus = ous.find((value) => value.id === formData.ou_id)
    const ousName = selectedOus === undefined ? '' : (`${i18n.language === 'en' ? selectedOus.en : selectedOus.nb} (${selectedOus.id})`)

    // TODO Fix this confusing mix between use of id and identifier
    const selectedRoleType = roleTypes.find((value) => value.id === (formData.role_type === undefined ? -1 : parseInt(formData.role_type, 10)))
    const roleTypeName = selectedRoleType === undefined ? '' : `${i18n.language === 'en' ? selectedRoleType.name_en : selectedRoleType.name_nb}`

    return (
      <>
        <Box sx={{
          paddingTop: '1rem',
          paddingBottom: '1rem',
        }}>
          {t('registerWizardText.summaryContactInformation')}
        </Box>

        <Box sx={{ typography: 'body2' }}>
          {t('registerWizardText.summaryPage')}
        </Box>
        <Box sx={{ marginTop: '1rem', maxWidth: '50rem' }}>
          <Box sx={{
            typography: 'subtitle2',
            marginBottom: '0.5rem',
            paddingLeft: '0.5rem',
            bgcolor: 'primary.dark',
          }}>{t('registerWizardText.contactInformation')}</Box>

          <Grid sx={{ typography: 'body2', paddingLeft: '0.5rem' }} container>
            <Grid item xs={4}>
              {t('input.fullName')}
            </Grid>
            <Grid item xs={8}>
              {formData.first_name} {formData.last_name}
            </Grid>
            <Grid item xs={4}>
              {t('input.email')}
            </Grid>
            <Grid item xs={8}>
              {formData.email}
            </Grid>
          </Grid>

          <Box sx={{
            paddingTop: '1rem',
            paddingBottom: '1rem',
          }}>
            {t('registerWizardText.summaryPeriod')}
          </Box>

          <Table>
            <TableHead sx={{
              paddingLeft: '0.5rem',
              bgcolor: 'primary.dark',
            }}>

              <TableRow>
                <TableCell>{t('registerWizardText.guestRole')}</TableCell>
                <TableCell>{t('registerWizardText.guestPeriod')}</TableCell>
                <TableCell>{t('registerWizardText.guestDepartment')}</TableCell>
                <TableCell>{t('input.comment')}</TableCell>
              </TableRow>
            </TableHead>
            <TableBody>
              <TableRow>
                <TableCell>{roleTypeName}</TableCell>
                <TableCell>{formData.role_start === null ? null : format(formData.role_start as Date, 'yyyy-MM-dd')} - {formData.role_end === null ? null : format(formData.role_end as Date, 'yyyy-MM-dd')}</TableCell>
                <TableCell>{ousName}</TableCell>
                <TableCell>{formData.comment}</TableCell>
              </TableRow>
            </TableBody>
          </Table>

          {/* TODO For now just showing a heading to give the user some feedback */}
          {submitState === SubmitState.SubmitSuccess && (
            <Box>
              <h2>Submit success</h2>
            </Box>
          )}

          {submitState === SubmitState.SubmitFailure && (
            <Box>
              <h2>Submit failure</h2>
            </Box>
          )}
        </Box>
      </>
    )
  },
)

export default StepSummary
