import React, { useEffect, useState } from 'react'
import {
  Table,
  TableBody,
  TableCell,
  TableContainer,
  TableHead,
  TableRow,
  Paper,
} from '@mui/material'

import Page from 'components/page'
import { useTranslation } from 'react-i18next'
import { Link } from 'react-router-dom'

type PersonInfo = {
  name: string
  role_nb: string
  role_en: string
  period: string
  active: boolean
  ou_nb: string
  ou_en: string
}

interface GuestProps {
  persons: PersonInfo[]
}

interface PersonLineProps {
  person: PersonInfo
}

interface FetchedPerson {
  first: string
  last: string
  role_nb: string
  role_en: string
  period: string
  ou_nb: string
  ou_en: string
  active: boolean
}

const PersonLine = ({ person }: PersonLineProps) => {
  const [t, i18n] = useTranslation(['common'])

  return (
    <TableRow
      key={person.name}
      sx={{ '&:last-child td, &:last-child th': { border: 0 } }}
    >
      <TableCell component="th" scope="row">
        {person.name}
      </TableCell>
      <TableCell align="left">
        {i18n.language === 'en' ? person.role_en : person.role_nb}
      </TableCell>
      <TableCell align="left">{person.period}</TableCell>
      <TableCell align="left">
        {i18n.language === 'en' ? person.ou_en : person.ou_nb}
      </TableCell>
      <TableCell align="left">
        <button type="button">{t('common:details')}</button>
      </TableCell>
    </TableRow>
  )
}

const ActiveGuests = ({ persons }: GuestProps) => {
  // Only show active people
  let guests = persons.length > 0 ? persons : []
  if (guests.length > 0) {
    guests = guests.filter((person) => person.active)
  }
  const [t] = useTranslation(['common'])
  return (
    <>
      <h1>{t('common:activeGuests')}</h1>
      <p>{t('common:activeGuestsDescription')}</p>
      <TableContainer component={Paper}>
        <Table sx={{ minWidth: 650 }} aria-label="simple table">
          <TableHead>
            <TableRow>
              <TableCell>{t('common:name')}</TableCell>
              <TableCell align="left">{t('common:role')}</TableCell>
              <TableCell align="left">{t('common:period')}</TableCell>
              <TableCell align="left">{t('common:ou')}</TableCell>
              <TableCell align="left">{t('common:choice')}</TableCell>
            </TableRow>
          </TableHead>
          <TableBody>
            {guests.map((person) => (
              <PersonLine person={person} />
            ))}

            <TableRow>
              <TableCell>
                {guests.length > 0 ? '' : t('common:noActiveGuests')}
              </TableCell>
            </TableRow>
          </TableBody>
        </Table>
      </TableContainer>
    </>
  )
}

const WaitingGuests = ({ persons }: GuestProps) => {
  // Only show non-active people
  let guests = persons.length > 0 ? persons : []
  if (guests.length > 0) {
    guests = guests.filter((person) => !person.active)
  }
  const [t] = useTranslation(['common'])

  return (
    <>
      <h1>{t('common:waitingGuests')}</h1>
      <p>{t('common:waitingGuestsDescription')}</p>

      <TableContainer component={Paper}>
        <Table sx={{ minWidth: 650 }} aria-label="simple table">
          <TableHead>
            <TableRow>
              <TableCell>{t('common:name')}</TableCell>
              <TableCell align="left">{t('common:role')}</TableCell>
              <TableCell align="left">{t('common:period')}</TableCell>
              <TableCell align="left">{t('common:ou')}</TableCell>
              <TableCell align="left">{t('common:choice')}</TableCell>
            </TableRow>
          </TableHead>
          <TableBody>
            {guests.map((person) => (
              <PersonLine person={person} />
            ))}
            <TableRow>
              <TableCell>
                {guests.length > 0 ? '' : t('common:noWaitingGuests')}
              </TableCell>
            </TableRow>
          </TableBody>
        </Table>
      </TableContainer>
    </>
  )
}

function FrontPage() {
  const [persons, setPersons] = useState<Array<PersonInfo>>([])

  const [t] = useTranslation(['common'])
  const fetchGuestsInfo = async () => {
    const response = await fetch('/api/ui/v1/guests/?format=json')
    const jsonResponse = await response.json()
    if (response.ok) {
      const roles = await jsonResponse.roles
      const guests: PersonInfo[] = roles.map((person: FetchedPerson) => ({
        name: `${person.first} ${person.last}`,
        role_nb: person.role_nb,
        role_en: person.role_en,
        period: person.period,
        active: person.active,
        ou_nb: person.ou_nb,
        ou_en: person.ou_en,
      }))
      setPersons(guests)
    }
  }

  useEffect(() => {
    fetchGuestsInfo()
  }, [])

  return (
    <Page header="Sponsor front page">
      <Link to="/register">{t('common:registerText')}</Link>
      <WaitingGuests persons={persons} />
      <hr className="rounded" />
      <ActiveGuests persons={persons} />
    </Page>
  )
}

export default FrontPage
