import datetime
from typing import Dict

import pytest
from rest_framework import status
from rest_framework.reverse import reverse
from rest_framework.status import HTTP_200_OK
from rest_framework.test import APIClient

from greg.models import (
    Identity,
    Sponsor,
    RoleType,
    OrganizationalUnit,
    ConsentType,
    Person,
    Role,
)


@pytest.fixture
def role_type_visiting_professor() -> RoleType:
    return RoleType.objects.create(
        identifier="visiting_professor",
        name_nb="Gjesteprofessor",
        name_en="Visiting professor",
        description_nb="Gjesteprofessor",
        description_en="Visiting professor",
        default_duration_days=180,
    )


@pytest.fixture
def unit_human_resources() -> OrganizationalUnit:
    return OrganizationalUnit.objects.create(
        orgreg_id="org_unit_1", name_nb="Personal", name_en="Human Resources"
    )


@pytest.fixture()
def sponsor_bar() -> Sponsor:
    return Sponsor.objects.create(feide_id="bar")


@pytest.fixture
def role_data_guest(
    role_type_test_guest: RoleType, sponsor_bar: Sponsor, unit_foo: OrganizationalUnit
) -> Dict:
    return {
        "type": "Test Guest",
        "start_date": "2021-06-10",
        "end_date": "2021-08-10",
        "sponsor_id": sponsor_bar.id,
        "orgunit_id": unit_foo.id,
    }


@pytest.fixture
def consent_foo() -> ConsentType:
    return ConsentType.objects.create(
        identifier="test_consent",
        name_en="Test1",
        name_nb="Test2",
        description_en="Test description",
        description_nb="Test beskrivelse",
        link_en="https://example.org",
        link_nb="https://example.org",
        valid_from="2018-01-20",
        user_allowed_to_change=True,
    )


@pytest.mark.django_db
def test_get_person(client, person_foo):
    resp = client.get(reverse("v1:person-detail", kwargs={"id": person_foo.id}))
    assert resp.status_code == HTTP_200_OK
    data = resp.json()
    assert data.get("id") == person_foo.id
    assert data.get("first_name") == person_foo.first_name
    assert data.get("last_name") == person_foo.last_name


@pytest.mark.django_db
def test_persons(client, person_foo, person_bar):
    resp = client.get(reverse("v1:person-list"))
    assert resp.status_code == HTTP_200_OK
    data = resp.json()
    assert len(data["results"]) == 2


@pytest.mark.django_db
def test_persons_verified_filter_include(
    client, person_bar, person_foo, person_foo_verified
):
    url = reverse("v1:person-list")
    response = client.get(url, {"verified": "true"})
    results = response.json()["results"]
    assert len(results) == 1
    assert results[0]["first_name"] == "Foo"
    assert results[0]["last_name"] == "Foo"


@pytest.mark.django_db
def test_persons_verified_filter_exclude(
    client, person_bar, person_foo, person_foo_verified
):
    url = reverse("v1:person-list")
    response = client.get(url, {"verified": "false"})
    results = response.json()["results"]
    assert len(results) == 1
    assert results[0]["first_name"] == "Bar"
    assert results[0]["last_name"] == "Bar"


@pytest.mark.django_db
def test_add_role(
    client, person_foo, role_type_visiting_professor, sponsor_guy, unit_human_resources
):
    url = reverse("v1:person_role-list", kwargs={"person_id": person_foo.id})
    roles_for_person = client.get(url).json()["results"]

    # Check that there are no roles for the person, and then add one
    assert len(roles_for_person) == 0

    role_data = {
        "type": "visiting_professor",
        "start_date": "2021-06-10",
        "end_date": "2021-08-10",
        "sponsor_id": "1",
        "orgunit_id": "1",
    }
    response = client.post(url, role_data)

    assert response.status_code == status.HTTP_201_CREATED

    response_data = response.json()
    roles_for_person = client.get(url).json()["results"]

    # Check that the role shows up when listing roles for the person now
    assert len(roles_for_person) == 1
    assert roles_for_person[0]["id"] == response_data["id"]


@pytest.mark.django_db
def test_update_role(client, person_foo, role_data_guest):
    url = reverse("v1:person_role-list", kwargs={"person_id": person_foo.id})
    response = client.post(url, role_data_guest)
    response_data = response.json()

    assert response_data["start_date"] == "2021-06-10"

    # Update the date and check that the change is registered
    role_id = response.json()["id"]
    updated_role = role_data_guest.copy()
    updated_role["start_date"] = "2021-06-15"

    url_detail = reverse(
        "v1:person_role-detail", kwargs={"person_id": person_foo.id, "id": role_id}
    )
    client.patch(url_detail, updated_role)

    updated_role_data = client.get(url)
    updated_data = updated_role_data.json()["results"][0]

    assert updated_data["id"] == role_id
    assert updated_data["start_date"] == "2021-06-15"


@pytest.mark.django_db
def test_delete_role(client, person_foo, role_data_guest):
    url = reverse("v1:person_role-list", kwargs={"person_id": person_foo.id})
    role_id = client.post(url, role_data_guest).json()["id"]
    roles_for_person = client.get(url).json()["results"]

    assert len(roles_for_person) == 1

    url_detail = reverse(
        "v1:person_role-detail", kwargs={"person_id": person_foo.id, "id": role_id}
    )
    client.delete(url_detail)

    assert len(client.get(url).json()["results"]) == 0


@pytest.mark.django_db
def test_identity_list(
    client, person_foo, person_foo_verified, person_foo_not_verified
):
    response = client.get(
        reverse("v1:person-list"),
        data={"first_name": person_foo.first_name, "last_name": person_foo.last_name},
    )
    person_id = response.json()["results"][0]["id"]
    response = client.get(
        reverse("v1:person_identity-list", kwargs={"person_id": person_id})
    )
    data = response.json()["results"]
    assert len(data) == 2

    first = data[0]
    assert first["person"] == 1
    assert first["type"] == "passport_number"
    assert first["source"] == "Test"
    assert first["value"] == "12345"
    assert first["verified"] == "manual"
    assert first["verified_by"] == 1
    # assert first["verified_at"] is a datetime string
    # assert first["created"] is a datetime string


@pytest.mark.django_db
def test_identity_add(client, person_foo):
    response = client.get(
        reverse("v1:person_identity-list", kwargs={"person_id": person_foo.id})
    )
    results = response.json()["results"]
    assert len(results) == 0

    data = {
        "type": Identity.IdentityType.FEIDE_ID,
        "source": "Test source",
        "value": "12345",
    }
    client.post(
        reverse("v1:person_identity-list", kwargs={"person_id": person_foo.id}),
        data=data,
    )

    response = client.get(
        reverse("v1:person_identity-list", kwargs={"person_id": person_foo.id})
    )
    results = response.json()["results"]
    assert len(results) == 1


@pytest.mark.django_db
def test_identity_add_duplicate_fails(client, person_foo, person_bar):
    data = {
        "type": Identity.IdentityType.FEIDE_ID,
        "source": "Test source",
        "value": "12345",
    }
    client.post(
        reverse("v1:person_identity-list", kwargs={"person_id": person_bar.id}),
        data=data,
    )

    response = client.post(
        reverse("v1:person_identity-list", kwargs={"person_id": person_foo.id}),
        data=data,
    )
    assert response.status_code == status.HTTP_400_BAD_REQUEST
    assert response.json() == {"non_field_errors": ["Identity already exists"]}


@pytest.mark.django_db
def test_identity_add_valid_duplicate(client, person_foo, person_bar):
    data = {
        "type": Identity.IdentityType.OTHER,
        "source": "Test source",
        "value": "12345",
    }
    client.post(
        reverse("v1:person_identity-list", kwargs={"person_id": person_bar.id}),
        data=data,
    )

    client.post(
        reverse("v1:person_identity-list", kwargs={"person_id": person_foo.id}),
        data=data,
    )


@pytest.mark.django_db
def test_identity_delete(client, person_foo):
    response = client.get(
        reverse("v1:person_identity-list", kwargs={"person_id": person_foo.id})
    )
    results = response.json()["results"]
    assert len(results) == 0

    data = {
        "type": Identity.IdentityType.FEIDE_ID,
        "source": "Test source",
        "value": "12345",
    }
    post_response = client.post(
        reverse("v1:person_identity-list", kwargs={"person_id": person_foo.id}),
        data=data,
    )
    identity_id = post_response.json()["id"]

    # Create two identities for the user
    data = {
        "type": Identity.IdentityType.PASSPORT_NUMBER,
        "source": "Test",
        "value": "1234413241235",
    }
    post_response2 = client.post(
        reverse("v1:person_identity-list", kwargs={"person_id": person_foo.id}),
        data=data,
    )
    identity_id2 = post_response2.json()["id"]

    response = client.get(
        reverse("v1:person_identity-list", kwargs={"person_id": person_foo.id})
    )
    results = response.json()["results"]
    assert len(results) == 2

    # Delete the first identity created
    client.delete(
        reverse(
            "v1:person_identity-detail",
            kwargs={"person_id": person_foo.id, "id": identity_id},
        )
    )

    # Check that the other identity is still there
    response = client.get(
        reverse("v1:person_identity-list", kwargs={"person_id": person_foo.id})
    )
    results = response.json()["results"]

    assert len(results) == 1
    assert results[0]["id"] == identity_id2


@pytest.mark.django_db
def test_identity_update(client, person_foo):
    response = client.get(
        reverse("v1:person_identity-list", kwargs={"person_id": person_foo.id})
    )
    results = response.json()["results"]
    assert len(results) == 0

    data = {
        "type": Identity.IdentityType.FEIDE_ID,
        "source": "Test source",
        "value": "12345",
    }
    client.post(
        reverse("v1:person_identity-list", kwargs={"person_id": person_foo.id}),
        data=data,
    )

    response = client.get(
        reverse("v1:person_identity-list", kwargs={"person_id": person_foo.id})
    )
    results = response.json()["results"]
    assert len(results) == 1

    identity_id = results[0]["id"]

    assert results[0]["type"] == data["type"]
    assert results[0]["source"] == data["source"]
    assert results[0]["value"] == data["value"]

    data_updated = {
        "type": Identity.IdentityType.PASSPORT_NUMBER,
        "source": "Test",
        "value": "10000",
    }
    patch_response = client.patch(
        reverse(
            "v1:person_identity-detail",
            kwargs={"person_id": person_foo.id, "id": identity_id},
        ),
        data=data_updated,
    )
    assert patch_response.status_code == status.HTTP_200_OK

    response = client.get(
        reverse("v1:person_identity-list", kwargs={"person_id": person_foo.id})
    )
    results = response.json()["results"]
    assert len(results) == 1

    assert results[0]["type"] == data_updated["type"]
    assert results[0]["source"] == data_updated["source"]
    assert results[0]["value"] == data_updated["value"]


@pytest.mark.django_db
def test_remove_person(
    client, person_foo, role_data_guest, person_foo_verified, person_foo_not_verified
):
    url = reverse("v1:person_role-list", kwargs={"person_id": person_foo.id})
    client.post(url, role_data_guest)

    roles_for_person = client.get(url).json()["results"]
    assert len(roles_for_person) == 1

    response = client.get(
        reverse("v1:person_identity-list", kwargs={"person_id": person_foo.id})
    )
    assert len(response.json()["results"]) == 2

    # Delete the person and check that the data has been removed
    client.delete(reverse("v1:person-detail", kwargs={"id": person_foo.id}))

    updated_role_data = client.get(url)
    assert len(updated_role_data.json()["results"]) == 0

    response = client.get(
        reverse("v1:person_identity-list", kwargs={"person_id": person_foo.id})
    )
    assert len(response.json()["results"]) == 0


@pytest.mark.skip("fails with IntegrityError")
@pytest.mark.django_db
def test_add_duplicate_role_fails(client, person_foo: Person, role_person_foo):
    url = reverse("v1:person_role-list", kwargs={"person_id": person_foo.id})
    roles_for_person = client.get(url).json()["results"]

    assert len(roles_for_person) == 1

    role_data = {
        "type": role_person_foo.type.identifier,
        "start_date": role_person_foo.start_date,
        "end_date": role_person_foo.end_date,
        "sponsor_id": role_person_foo.sponsor_id.id,
        "orgunit_id": role_person_foo.unit_id,
    }
    response = client.post(url, role_data)
    # If the role cannot be create the return code is 400
    assert response.status_code == status.HTTP_400_BAD_REQUEST
    # TODO(jbr): assert response.json() explains integrity error
    assert False

    # Check that there is still only one role attached to the person
    roles_for_person = client.get(url).json()["results"]
    assert len(roles_for_person) == 1


@pytest.mark.django_db
def test_filter_active_includes_person_with_active_role(
    client,
    person_foo: Person,
    sponsor_guy: Sponsor,
    unit_foo: OrganizationalUnit,
    role_type_test_guest: Role,
):
    date_today = datetime.date.today()

    Role.objects.create(
        person=person_foo,
        type=role_type_test_guest,
        end_date=date_today + datetime.timedelta(days=1),
        sponsor_id=sponsor_guy,
        orgunit_id=unit_foo,
    )

    url = reverse("v1:person-list")
    response = client.get(url, {"active": "true"})
    results = response.json()["results"]
    assert len(results) == 1
    assert results[0]["first_name"] == "Foo"
    assert results[0]["last_name"] == "Foo"


@pytest.mark.django_db
def test_filter_active_excludes_person_with_no_active_role(
    client, person_foo: Person, role_person_foo: Role
):
    url = reverse("v1:person-list")
    response = client.get(url, {"active": "true"})
    results = response.json()["results"]
    assert len(results) == 0


@pytest.mark.django_db
def test_filter_active_value_false(
    client: APIClient,
    person_foo: Person,
    role_type_test_guest: Role,
    sponsor_guy: Sponsor,
    unit_foo: OrganizationalUnit,
):
    """Check that filter is not used when specified to False"""
    date_today = datetime.date.today()

    # Inactive role
    Role.objects.create(
        person=person_foo,
        type=role_type_test_guest,
        end_date=date_today - datetime.timedelta(days=1),
        sponsor_id=sponsor_guy,
        orgunit_id=unit_foo,
    )

    url = reverse("v1:person-list")
    response = client.get(url, {"active": False})
    results = response.json()["results"]
    assert len(results) == 1
