import json
import datetime
from uuid import uuid4
from django.core import exceptions
from django.db import transaction
from django.http.response import JsonResponse

from django.utils import timezone
from rest_framework import serializers, status
from rest_framework.authentication import SessionAuthentication, BasicAuthentication
from rest_framework.generics import CreateAPIView
from rest_framework.parsers import JSONParser
from rest_framework.permissions import AllowAny, IsAuthenticated
from rest_framework.response import Response
from rest_framework.views import APIView

from greg.models import Identity, Invitation, InvitationLink, Person, Role, Sponsor
from greg.permissions import IsSponsor
from gregui.api.serializers.guest import GuestRegisterSerializer
from gregui.api.serializers.invitation import InviteGuestSerializer


from gregui.models import GregUserProfile


class CreateInvitationView(CreateAPIView):
    """
    Invitation creation endpoint


    {
        "first_name": "dfff",
        "last_name": "sss",
        "date_of_birth": null,
        "role": {
            "orgunit_id": 1,
            "start_date": null,
            "type": 1,
            "end_date": "2021-12-15",
            "contact_person_unit": "",
            "comments": "",
            "available_in_search": false
        }
    }
    """

    authentication_classes = [BasicAuthentication, SessionAuthentication]
    permission_classes = [IsSponsor]
    parser_classes = [JSONParser]
    serializer_class = InviteGuestSerializer

    def post(self, request, *args, **kwargs) -> Response:
        """
        Invitation creation endpoint

        Restricted to Sponsors, and a sponsor can only invite guests to OUs
        they are registered to.

        The next step in the flow is for the guest to use their link, review the
        information, and confirm it.
        """
        sponsor_user = GregUserProfile.objects.get(user=request.user)
        serializer = self.serializer_class(
            data=request.data, context={"request": request}
        )
        serializer.is_valid(raise_exception=True)
        person = serializer.save()

        invitationlink = InvitationLink.objects.filter(
            invitation__role__person=person.id,
            invitation__role__sponsor_id=sponsor_user.sponsor,
        )
        # TODO: send email to invited guest
        print(invitationlink)
        return Response(status=status.HTTP_201_CREATED)


class CheckInvitationView(APIView):
    authentication_classes = []
    permission_classes = [AllowAny]

    def get(self, request, *args, **kwargs):
        """
        Endpoint for verifying and setting invite_id in session.

        This endpoint is meant to be called in the background by the frontend on the
        page you get to by following the invitation url to the frontend. This way a
        session is created keeping the invite id safe, until the user returns from
        feide login if they choose to use it.
        """
        invite_id = kwargs["uuid"]
        try:
            invite_link = InvitationLink.objects.get(uuid=invite_id)
        except (InvitationLink.DoesNotExist, exceptions.ValidationError):
            return Response(status=status.HTTP_403_FORBIDDEN)
        if invite_link.expire <= timezone.now():
            return Response(status=status.HTTP_403_FORBIDDEN)
        request.session["invite_id"] = invite_id
        return Response(status=status.HTTP_200_OK)


class InvitedGuestView(APIView):
    authentication_classes = [SessionAuthentication, BasicAuthentication]
    permission_classes = [AllowAny]
    parser_classes = [JSONParser]
    serializer_class = GuestRegisterSerializer

    def get(self, request, *args, **kwargs):
        """
        Endpoint for fetching data related to an invite

        Used by the frontend for fetching data from the backend for review by a guest
        in the frontend, before calling the post endpoint defined below with updated
        info and confirmation.
        """
        invite_id = request.session.get("invite_id")
        try:
            invite_link = InvitationLink.objects.get(uuid=invite_id)
        except (InvitationLink.DoesNotExist, exceptions.ValidationError):
            return Response(status=status.HTTP_403_FORBIDDEN)
        if invite_link.expire <= timezone.now():
            return Response(status=status.HTTP_403_FORBIDDEN)

        # if invite_id:
        invite_link = InvitationLink.objects.get(uuid=invite_id)
        role = invite_link.invitation.role
        person = role.person
        sponsor = role.sponsor_id

        try:
            fnr = person.identities.get(type="norwegian_national_id_number").value
        except Identity.DoesNotExist:
            fnr = None
        try:
            passport = person.identities.get(type="passport_number").value
        except Identity.DoesNotExist:
            passport = None

        data = {
            "person": {
                "first_name": person.first_name,
                "last_name": person.last_name,
                "email": person.email,
                "mobile_phone": person.mobile_phone,
                "fnr": fnr,
                "passport": passport,
            },
            "sponsor": {
                "first_name": sponsor.first_name,
                "last_name": sponsor.last_name,
            },
            "role": {
                "ou_name_nb": role.orgunit_id.name_nb,
                "ou_name_en": role.orgunit_id.name_en,
                "role_name_nb": role.type.name_nb,
                "role_name_en": role.type.name_en,
                "start": role.start_date,
                "end": role.end_date,
                "comments": role.comments,
            },
        }
        return JsonResponse(data=data, status=status.HTTP_200_OK)

    def post(self, request, *args, **kwargs):
        """
        Endpoint for confirmation of data updated by guest

        Used by frontend when the confirm button at the end of the review flow is
        clicked by the user. The next part of the flow is for the sponsor to confirm
        the guest.
        """
        invite_id = kwargs["uuid"]
        data = request.data

        with transaction.atomic():
            # Ensure the invitation link is valid and not expired
            try:
                invite_link = InvitationLink.objects.get(uuid=invite_id)
            except (InvitationLink.DoesNotExist, exceptions.ValidationError):
                return Response(status=status.HTTP_403_FORBIDDEN)
            if invite_link.expire <= timezone.now():
                return Response(status=status.HTTP_403_FORBIDDEN)

            # Get objects to update
            person = invite_link.invitation.role.person

            # Update with input from the guest
            mobile = data.get("mobile_phone")
            if mobile:
                person.mobile_phone = data["mobile_phone"]

            # Mark guest interaction done
            person.registration_completed_date = timezone.now().date()
            person.save()

            # Expire the invite link
            invite_link.expire = timezone.now()
            invite_link.save()
            # TODO: Send an email to the sponsor?
        return Response(status=status.HTTP_201_CREATED)
